;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xc853f52da42847259b0df1ebe2064980 . "./code.cg2js/cg-acache.fasl")
(#x51d4a648f4d2c50cc850d7cdda93e5d3 . "./code.cg2js/cg-base.fasl")
(#x6acf03fd16400d727b1e21e49817c316 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x3b36cd64b5e6047262d3314bc4bdac71 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xe238cbf56a3901b35d56cd0c6fa758b7 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x96761bdcb3e5860e40a122e8eecfdd87 . "./code.cg2js/cg-button.fasl")
(#x9f15ca90ae57a5d34b8044629b53475a . "./code.cg2js/cg-calendar.fasl")
(#x6bd6a70645d1a1db6cf5e6fb86af8c70 . "./code.cg2js/cg-caret.fasl")
(#x7eefd8fc6053654ecd2f5c22594846ae . "./code.cg2js/cg-chart-or-plot.fasl")
(#x948c0db5ecb61f11f1dc773abaef047d . "./code.cg2js/cg-chart-widget.fasl")
(#x01de1e93d786b4cfef8a707d767c15f8 . "./code.cg2js/cg-check-box.fasl")
(#xfa2ab1b1bd8c6f0c6943df1c2942c8d0 . "./code.cg2js/cg-choice-list.fasl")
(#x70b68c694b19aa93833dddd66f57a37a . "./code.cg2js/cg-choose-printer.fasl")
(#x75828f2e6657b5205faed7920ddfd753 . "./code.cg2js/cg-class-grid.fasl")
(#x4bcab740f48ada583977eeaf43dfb549 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x67e4d90893df2e70f90dd1e568840614 . "./code.cg2js/cg-class-support.fasl")
(#x3ccffc859bf3f3ebfdcea8bd0c118d3b . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x3aa8c906abb3a32808b3e14fa548c7a6 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x2dc2b4932098986be7e079ff7b6e6034 . "./code.cg2js/cg-clipboard.fasl")
(#x02b2f14d4dedf25677568c0167062a83 . "./code.cg2js/cg-color-dialog.fasl")
(#x469faa31164bb9736adca43af791dd95 . "./code.cg2js/cg-combo-box.fasl")
(#x3ea391339e95cbcea426039aa53e5cea . "./code.cg2js/cg-common-control.fasl")
(#x9960e534b6c627bee873624d3332f14b . "./code.cg2js/cg-comtab.fasl")
(#x8baf4451a907d8d55ac5bb2bf9c92bef . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x81f4f4e64864819a2063746e14bea9a3 . "./code.cg2js/cg-curve.fasl")
(#xe5905a9988ab3bf19cea79cfdb85feb6 . "./code.cg2js/cg-dde-utils.fasl")
(#xf40c4ef24351e264e83d6dad7924f9f4 . "./code.cg2js/cg-dev.fasl")
(#x7bcf86847aeafef1d27a068d391a7906 . "./code.cg2js/cg-dialog-item.fasl")
(#xf8bbfc10b5a35693134e0b5723e2e8da . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x30b4f8f4e81d8a11b2b22e3bb3e9a080 . "./code.cg2js/cg-directory-dialog.fasl")
(#x894402701132c3c658e3a2ddf529023d . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xc4304bb4c6d937c6cdd84532648c857d . "./code.cg2js/cg-drag-and-drop.fasl")
(#xe3204c29b9141fd7916445b873caf8b2 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x24f87353b94c48978ab1ce5105f7e685 . "./code.cg2js/cg-drawable.fasl")
(#x523224604d07e56dc63e52b8d8ebcc5e . "./code.cg2js/cg-dropping-outline.fasl")
(#x0c4c718732ce394e2840decff247d736 . "./code.cg2js/cg-edit-in-place.fasl")
(#xdade389118e9402eb203c87a0484ba72 . "./code.cg2js/cg-editable-text.fasl")
(#x3c4ed333b91b7baf67d5177db65bb785 . "./code.cg2js/cg-file-dialog.fasl")
(#x6b8fb2b137308a408ace81d108585844 . "./code.cg2js/cg-fill-texture.fasl")
(#xa3a9b953c22df2e11c6c4c7ca44e66b9 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x7b8394db755492843645ffce44384ebd . "./code.cg2js/cg-font-dialog.fasl")
(#xcb78a641e0240dfad405d6bcfe6c1dcf . "./code.cg2js/cg-gesture-emulation.fasl")
(#x7203e0890497f42aef1a5aa583232448 . "./code.cg2js/cg-get-pixmap.fasl")
(#x7206c9d69460decea07d98154f233e6e . "./code.cg2js/cg-get-position.fasl")
(#x7c3f6e5aa02e461b8d4501ec57cb7b0c . "./code.cg2js/cg-graphics-context.fasl")
(#xeb2b2f3a872f116b14293c2d0b7b24c3 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7cd8763997b7da2dcf846bd2f21defee . "./code.cg2js/cg-grid-widget.fasl")
(#xa92be25aea0571ba90e60363da998b93 . "./code.cg2js/cg-group-box.fasl")
(#x5101343b2c8b2d10f4de8f531682544e . "./code.cg2js/cg-header-control.fasl")
(#xf261b1aee0fe954ece54cf6ea2be51cb . "./code.cg2js/cg-hotspot.fasl")
(#x549bfba11de7cbd9cbc7a8f368e6e516 . "./code.cg2js/cg-html-dialog.fasl")
(#x1f4fd43f5bd1f4cbb50710ad406cf015 . "./code.cg2js/cg-html-widget.fasl")
(#xaf5e4f045dd6e7b1676dfe00165540cd . "./code.cg2js/cg-icon-pixmap.fasl")
(#x853d1ada510782d6ac34501f10f2ae41 . "./code.cg2js/cg-icon.fasl")
(#xdaf5e14ccfd83348741209e440ba4436 . "./code.cg2js/cg-ie.fasl")
(#x6934f721c0ffa646a690c704ff04e7c0 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x22cf7d630cee79c33656b837510523d0 . "./code.cg2js/cg-intersect.fasl")
(#xae00a34c7bddb43071ae510b31422da3 . "./code.cg2js/cg-item-list.fasl")
(#x284aa62ebc26734b7bf6d19b2daefb22 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x43460cd337d7e2e644a08900fe3f3bba . "./code.cg2js/cg-lamp.fasl")
(#x0f5dbce25fe8442ee6fab139936ac53f . "./code.cg2js/cg-layout.fasl")
(#x280c52c8851870d05d4ac76075fd6eed . "./code.cg2js/cg-lettered-menu.fasl")
(#xe52ff06376f55bdb231bf49d18459938 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x3d7fbf4494b57fc8129a33f58b78671a . "./code.cg2js/cg-lisp-text.fasl")
(#x71455debffca78ffdbc7493fe0112957 . "./code.cg2js/cg-lisp-widget.fasl")
(#xfe97b4da27947c9e78aab75d5de5e976 . "./code.cg2js/cg-list-view.fasl")
(#xbfe8c39944be84f78a84b9edc7f826bf . "./code.cg2js/cg-mci.fasl")
(#x2a68d0c293f7dd5360745d7289ae3331 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x5da94ac1cd5778c52e53b45d0871b8e6 . "./code.cg2js/cg-menu.fasl")
(#x658e5eb378b13953c4c56060bd94a8da . "./code.cg2js/cg-message-dialog.fasl")
(#x13ee9fbf9b876d111ad0ae699027af6e . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x94c2c1c582b66c0b267f71d2aeeec33a . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xeed80bcfd64f64b9a32fc46ab590e8eb . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xe9efd26f96e8eebe63d3a34d1558bd87 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa0a37eee1bbdcf1da6271d5ebc4b6037 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xd123c9d591213b740a2368d99ac97ef5 . "./code.cg2js/cg-nodes.fasl")
(#xc35c7b85a2b3ecf639de69f94954e3a9 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x7009350daab4db268a06326ea1062629 . "./code.cg2js/cg-object-editor.fasl")
(#x8a290fce330c437a64b0e37ede3e917d . "./code.cg2js/cg-ocx.fasl")
(#xdf021658e1a8b459446ebfd09774f71c . "./code.cg2js/cg-os-widget.fasl")
(#xf8f932d3246605e38445feb32bd35dcd . "./code.cg2js/cg-os-window.fasl")
(#xf137c3d497b5f4a379692f5431996b41 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xa998e1ad90ceb48ae367d637481db33b . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xf05600260f525db00abd31d7263d87f8 . "./code.cg2js/cg-outline.fasl")
(#x89cfe865fb0266ddf4dbe8e30eff1a8d . "./code.cg2js/cg-palette.fasl")
(#xa7159271cb628a1251bdac6941241434 . "./code.cg2js/cg-paren-matching.fasl")
(#x2e3e6f93ccbe6054d5b02b6ba5c8cbe4 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc496b23254f65ba0de0ae03a33b2c2c3 . "./code.cg2js/cg-picture-widget.fasl")
(#x6b1d3c8de2c9ba07e2d83ef02cc84be6 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x788da86d8de1abbcd94dbfc3617dddbb . "./code.cg2js/cg-pixmap-printing.fasl")
(#x416c3a0d2f975ba34b9a0b8152ae28ad . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4148345bac36a14c86242881149b41b2 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xff1a423d4df69df50341c61033c81de1 . "./code.cg2js/cg-pixmap.fasl")
(#x3291186f627724e25c6527d26e30764a . "./code.cg2js/cg-plot-widget.fasl")
(#x54d705c9b90820ca3d93f80e305204cc . "./code.cg2js/cg-printing.fasl")
(#x6d1f7ec80c20abd407abcc2797d6f102 . "./code.cg2js/cg-progress-indicator.fasl")
(#xed9558312538b26ce0b39458d0dd29a9 . "./code.cg2js/cg-project-window.fasl")
(#x1c6f02a1de7464e33809980da7356902 . "./code.cg2js/cg-property.fasl")
(#xe6b71e51101007d782a47fd749084630 . "./code.cg2js/cg-radio-button.fasl")
(#x755877bb1a0da27e343b989d5c8f9ac1 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xfd1612bc3c469dcad76f448f94a4bda6 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xa64acac2b0bb1e310dff2a35179d3ff0 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x98fbec1084cb9c3686a066be990234ac . "./code.cg2js/cg-rich-edit.fasl")
(#x90f0f333ccf03ac35493654e74bb5266 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xec5440ee059812e473d20bb38def7136 . "./code.cg2js/cg-scaling-stream.fasl")
(#x432a8dadd80960870dbe81f8d8e0e1d1 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x4a3fa756993c677f0efa381e9b6436d5 . "./code.cg2js/cg-scroll-bar.fasl")
(#xbbf5774fef0bb8ba14dfe35544364c5d . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xda5dda5c2353a9a44f179345a72bc2fb . "./code.cg2js/cg-selected-object.fasl")
(#xefca79bcedd9a9bcbacbb8933b696131 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xe5f2e914dbe819bd2e0b7f593cdd3209 . "./code.cg2js/cg-split-bar.fasl")
(#xd722b0b3e68f0459158dd6ebef0382e6 . "./code.cg2js/cg-static-text.fasl")
(#x58efe84a6253d844ade1ecf1c110c417 . "./code.cg2js/cg-status-bar.fasl")
(#x85835473eb4bb1da7a68906462dc27d7 . "./code.cg2js/cg-string-dialog.fasl")
(#x06a11b6f2192bdc869d531257978216c . "./code.cg2js/cg-tab-control.fasl")
(#x7156845cc1a8e7220126b5968e601da6 . "./code.cg2js/cg-template-string.fasl")
(#xac1efad40a61fc70fe47f73107ddf3c0 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x8ba5e981201443916505043472e97ec1 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x0e39bd782766e70c809be26f1b2b57fb . "./code.cg2js/cg-text-edit-pane.fasl")
(#x85c6eadaa4314e505fc748450fcedc94 . "./code.cg2js/cg-text-or-combo.fasl")
(#x4448d428e8746397da3a4c09efb421d1 . "./code.cg2js/cg-text-widget.fasl")
(#xf30999017ce5da64c2fc176ebe7adbea . "./code.cg2js/cg-timer.fasl")
(#xdc939acad80bfc411dbc383d992edbba . "./code.cg2js/cg-toggling-widget.fasl")
(#xc5533e7a473cf536b69aab54bd9cdc5f . "./code.cg2js/cg-toolbar.fasl")
(#x019e7e4c087df5b8a3ce8b2db1589732 . "./code.cg2js/cg-tooltip.fasl")
(#xe16da54cb3b0dba80f9cbf12c01f7468 . "./code.cg2js/cg-touch.fasl")
(#x22fab1050900ac4259a0318e9b9672d3 . "./code.cg2js/cg-trackbar.fasl")
(#x9369fe3b420c6c526c09e73913dcecda . "./code.cg2js/cg-tray.fasl")
(#x77981f02abdb5978d806cc77a537090b . "./code.cg2js/cg-up-down-control.fasl")
(#x2db839d3eaa46cc917668f8030331997 . "./code.cg2js/cg-utility-dialog.fasl")
(#x8074cc2dcc36ebde7e8155c660963485 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x043cb9258e3833a2a33c523f1d2c5ffb . "./code.cg2js/cg-web-browser.fasl")
(#x5311b6b0c86fc985fe014faa32936b86 . "./code.cg2js/cg-wrap-string.fasl")
(#x75f63ea02407b49e50374f6dd1560390 . "./code.cg2js/cg-yes-no-list.fasl")
(#x571ccae89bc2e1eea4bc03bc62ab6a2a . "./code.cg2js/cg-yes-no-string.fasl")
(#x9ca87f38fb94847a6a970b887df3f1e6 . "./code.cg2js/cg.fasl")
(#x18137e9c1d60aac4071956e28882f507 . "./code.idejs/ide-acache.fasl")
(#xd116d198731eddecb78b57ea6ec67f56 . "./code.idejs/ide-apropos.fasl")
(#xd70edf5bcdedbe5afa002feba3b22f60 . "./code.idejs/ide-base.fasl")
(#xa22bacd0c88a0ccbae779da950f189b9 . "./code.idejs/ide-class-browser.fasl")
(#x61ad7717462e1c2384fbe89600909700 . "./code.idejs/ide-class-grapher.fasl")
(#x687d800b345e337221d3d380db6fd2bd . "./code.idejs/ide-clipboard.fasl")
(#xcc3d0ff34a6bd4811720463f2b21b6ab . "./code.idejs/ide-dev.fasl")
(#xc37075e0025c66d5193b591d907ee4e6 . "./code.idejs/ide-editor.fasl")
(#xe08b9523d3f40a52b41fbe7b4cc5d445 . "./code.idejs/ide-find-definitions.fasl")
(#xdf5dbcf9f1a7c41cfc5c53cfe8a5a164 . "./code.idejs/ide-find-in-files.fasl")
(#xc8a74c57d246f2f033907120f2ad2f0d . "./code.idejs/ide-form.fasl")
(#xbc19e5e8aadbba4f5926f92996c0bec4 . "./code.idejs/ide-grapher.fasl")
(#x12466d4f633ba55169f346f8fed0fff8 . "./code.idejs/ide-inspector.fasl")
(#x3ceb212839371c24a4410470cb78ca19 . "./code.idejs/ide-intro.fasl")
(#xc5f21a095b7e48ef62d5f8fb1294709b . "./code.idejs/ide-keyshort.fasl")
(#x5d0c0519118de8bd19e13dec96adad1a . "./code.idejs/ide-navigator.fasl")
(#x732305390792263412552f020f3e2df4 . "./code.idejs/ide-package-list.fasl")
(#x49a8aebe22c38ab08750fd1bab79eb13 . "./code.idejs/ide-patch-dialog.fasl")
(#x2336ee710d2ebfc3411a54bdc03b442f . "./code.idejs/ide-process-browser.fasl")
(#xdb2bd6dcf2d0d8c696f646fb1713fd6b . "./code.idejs/ide-project.fasl")
(#x78a5a65baf6a438812d0e13e8e12b896 . "./code.idejs/ide-trace.fasl")
(#xad45334f22c4eb8ec3da194a8330717a . "./code.idejs/ide-window-list.fasl")
(#x325e8b3970a679bf62a3ea6c15e20442 . "./code.idejs/ide.fasl")
(#xf456f3c7874f303a35ad67e1f3673be9 . "./cgjs.html")
(#x85aa0e1902236a67f67453f2c909ea34 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
